@@ -36,6 +36,7 @@
 #include "recovery.h"
 #include "rgrp.h"
 #include "super.h"
+#include "mount.h"
 
 /**
  * gfs_write_inode - Make sure the inode is stable on the disk
@@ -319,9 +320,28 @@ gfs_remount_fs(struct super_block *sb, int *flags, char *data)
 {
 	struct gfs_sbd *sdp = get_v2sdp(sb);
 	int error = 0;
+	struct gfs_args *args;
 
 	atomic_inc(&sdp->sd_ops_super);
 
+	args = kmalloc(sizeof(struct gfs_args), GFP_KERNEL);
+	if (!args)
+		return -ENOMEM;
+
+	error = gfs_make_args(data, args);
+	if (error) {
+		printk("GFS: can't parse remount arguments\n");
+		goto out;
+	}
+	if (args->ar_posix_acls) {
+		sdp->sd_args.ar_posix_acls = TRUE;
+		sb->s_flags |= MS_POSIXACL;
+	}
+	else {
+		sdp->sd_args.ar_posix_acls = FALSE;
+		sb->s_flags &= ~MS_POSIXACL;
+	}
+
 	if (*flags & (MS_NOATIME | MS_NODIRATIME))
 		set_bit(SDF_NOATIME, &sdp->sd_flags);
 	else
@@ -338,6 +358,8 @@ gfs_remount_fs(struct super_block *sb, int *flags, char *data)
 	/*  Don't let the VFS update atimes.  GFS handles this itself. */
 	*flags |= MS_NOATIME | MS_NODIRATIME;
 
+out:
+	kfree(args);
 	return error;
 }
 
